==> luna_lambda_tools-0.1.49-py3-none-any.whl: luna_lambda_tools/private/base_application_tasks.py <==
import asyncio
import importlib
from typing import Awaitable, final
from uuid import uuid4

from cow.monitoring.manager import LunaMonitoringManager
from cow.utils.check_connection import checkConnectionToLunaImageStore
from cow.utils.healthcheck import checkService
from luna3.common.base_api import BaseHTTPAPI
from luna3.settings import HTTPSettings

from .base_application import BaseAppCtx, BaseLambdaApp, Luna3Session as _Luna3Session, getServiceClientKwargs
from .task_unit_config import ServiceSettings
from .tasks_tools.base_subtask import BaseSubTask, SubTaskToImageStore
from .tasks_tools.monitoring.points import SubtaskMonitoringBasePoint, WorkerRequestPoint
from .tasks_tools.redis_db.context import WorkerRedisContext


class Luna3Session(_Luna3Session):
    """
    Initializable luna3 sessions.
    """

    def __init__(self, settings: ServiceSettings, lambdaType: str):
        self.settings = settings
        super().__init__(settings=settings, lambdaType=lambdaType)

    def _getLuna3Kwargs(self) -> dict[str, HTTPSettings]:
        kwargs = super()._getLuna3Kwargs()
        if self.settings.additionalServicesUsage.lunaImageStore:
            kwargs["tasksResultStoreSettings"] = HTTPSettings(
                **getServiceClientKwargs(
                    addressSetting=self.settings.taskResultStorage,
                    timeoutSettings=self.settings.taskResultStorageTimeouts,
                )
            )
        if self.settings.additionalServicesUsage.lunaHandlers:
            kwargs["handlersSettings"] = HTTPSettings(
                self.settings.handlersAddress.apiVersion,
                self.settings.handlersAddress.origin,
                **self.settings.handlersTimeouts.asDict(),
                asyncRequest=True,
            )
        return kwargs

    def getRuntimeChecks(self, includeLunaServices: bool = False) -> list[tuple[str, Awaitable]]:
        """
        Returns configured system checks, pairs of (name, coroutine).

        Warnings:
            Check license session in licenseChecker

        Args:
            includeLunaServices: A bool, whether to include checks for luna services.
        """
        checks = super().getRuntimeChecks(includeLunaServices=includeLunaServices)
        if self.tasksResultsStoreSession:
            taskResultClient = self.tasksResultsStoreSession.getClient()
            checks.append(("task", checkService(taskResultClient, includeLunaServices)))
        return checks

    async def probe(self) -> bool:
        """Check that we can connect to the services with current configuration. Can be used without initialization"""

        def generateCheck(client: BaseHTTPAPI, checker, postfix: str = "", **kwargs):
            check = checker(
                client.origin,
                client.api,
                totalTimeout=client.totalTimeout,
                connectTimeout=client.connectTimeout,
                sockReadTimeout=client.sockReadTimeout,
                sockConnectTimeout=client.sockConnectTimeout,
                asyncCheck=True,
                postfix=postfix,
                **kwargs,
            )

            return check

        checks = self.getProbeChecks()
        if self.tasksResultsStoreSession:
            taskResultClient = self.tasksResultsStoreSession.getClient()
            checks.append(
                generateCheck(
                    taskResultClient,
                    checkConnectionToLunaImageStore,
                    postfix="task result",
                    bucket=self.settings.taskResultStorage.bucket,
                )
            )
        return all(await asyncio.gather(*checks))


class AppCtx[TConfig: ServiceSettings](BaseAppCtx):
    """Configurator application context"""

    #: worker redis db context
    workerRedisDBContext: WorkerRedisContext | None
    #: worker tasks
    tasks: dict[BaseSubTask, asyncio.Task]
    #: worker separation tasks
    separationTasks: dict[BaseSubTask, asyncio.Task]
    #: lambda type
    lambdaType: str = "tasks"
    #: asyncio tasks
    tasks: dict
    #: separationTasks: dict
    separationTasks: dict
    #: luna 3 session
    luna3Session: Luna3Session | None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.tasks = {}
        self.separationTasks = {}
        self.luna3session = Luna3Session(settings=self.serviceConfig, lambdaType=self.lambdaType)

    @final
    async def initialize(self):
        """Description see :func:`~_LunaApplication.initialize`"""
        await super().initialize()

        if self.serviceConfig.additionalServicesUsage.lunaImageStore:
            self.initializeSubTaskToImageStore(settings=self.serviceConfig)

        self.initializeBaseWorkerClass(settings=self.serviceConfig, monitoring=self.monitoring)
        self.initializeMonitoringPoints(serviceName=self.lambdaName)
        self.workerRedisDBContext = WorkerRedisContext(self.serviceConfig.redisDB)

        kwargs = {
            "redisContext": self.workerRedisDBContext,
            "luna3Client": self.luna3session.getClient(str(uuid4())),
            "app": self,
        }
        try:
            lambdaTaskClass = importlib.import_module("lambda_main").LambdaTask
            tsk = lambdaTaskClass(**kwargs)
            self.tasks[tsk] = asyncio.create_task(tsk.start(lambdaId=self.lambdaName))
            self.separationTasks[tsk] = asyncio.create_task(tsk.separationStart(lambdaId=self.lambdaName))
        except Exception as exc:
            excArgs = " ".join(getattr(exc, "args", []))
            raise RuntimeError(f"Lambda tasks initialize error: {str(exc)} {excArgs}")

    @final
    async def shutdown(self):
        await super().shutdown()

        await self.workerRedisDBContext.close()

        for task, asyncioTask in self.tasks.items():
            await task.onShutDown()
            asyncioTask.cancel()
        for asyncioTask in self.separationTasks.values():
            asyncioTask.cancel()

    @staticmethod
    def initializeSubTaskToImageStore(settings: ServiceSettings):
        """
        Initialize SubTask to image store class.
        Args:
            settings: task worker settings
        """
        SubTaskToImageStore.initFromConfig(
            resultStoreBucket=settings.taskResultStorage.bucket,
            faceSampleBucket=settings.faceSamplesStorage.bucket,
            bodySampleBucket=settings.bodySamplesStorage.bucket,
            imageOriginBucket=settings.imageOriginStorage.bucket,
            taskResultStorageTimeouts=settings.taskResultStorageTimeouts,
            faceSamplesStorageTimeouts=settings.faceSamplesStorageTimeouts,
            bodySamplesStorageTimeouts=settings.bodySamplesStorageTimeouts,
            imageOriginStorageTimeouts=settings.imageOriginStorageTimeouts,
            imageStoreRequestConcurrency=settings.tasksToImageStoreRequestsConcurrency,
            taskExternalArchiveTimeouts=settings.taskExternalArchiveTimeouts,
        )

    @staticmethod
    def initializeBaseWorkerClass(settings: ServiceSettings, monitoring: LunaMonitoringManager | None) -> None:
        """
        Initialize base SubTask worker class.
        Args:
            settings: task worker settings
            monitoring: luna influx monitoring
        """
        BaseSubTask.initFromConfig(
            maxErrorCountPerTask=settings.maxErrorCountPerTask,
            influxMonitoring=monitoring,
            limits=settings.platformLimits,
            facesRequestsConcurrency=settings.tasksToFacesRequestsConcurrency,
            handlersRequestConcurrency=settings.tasksToHandlersConcurrency,
            matcherRequestConcurrency=settings.tasksToMatcherConcurrency,
            externalApiAddress=settings.externalApiAddress,
            lisObjectsAddress=settings.objectsOriginStorage,
            lisEnabled=settings.additionalServicesUsage.lunaImageStore,
        )

    @staticmethod
    def initializeMonitoringPoints(serviceName: str):
        """
        Initialize monitoring points classes.
        Args:
            serviceName: custom service name
        """
        WorkerRequestPoint.initialize(serviceName=serviceName)
        SubtaskMonitoringBasePoint.initialize(serviceName=serviceName)


class LambdaTasksApp(BaseLambdaApp):
    """Tasks lambda app"""

    # service context
    ctxCls = AppCtx
    # service config
    configCls = ServiceSettings

==> luna_lambda_tools-0.1.78-py3-none-any.whl: luna_lambda_tools/private/base_application_tasks.py <==
import asyncio
import importlib
from typing import Awaitable, final
from uuid import uuid4

from cow.monitoring.manager import LunaMonitoringManager
from cow.utils.check_connection import checkConnectionToLunaImageStore
from cow.utils.healthcheck import checkService
from luna3.common.base_api import BaseHTTPAPI
from luna3.settings import HTTPSettings

from .base_application import BaseAppCtx, BaseLambdaApp, Luna3Session as _Luna3Session, getServiceClientKwargs
from .task_unit_config import ServiceSettings
from .tasks_tools.base_subtask import BaseSubTask, SubTaskToImageStore
from .tasks_tools.monitoring.points import SubtaskMonitoringBasePoint, WorkerRequestPoint
from .tasks_tools.redis_db.context import WorkerRedisContext


class Luna3Session(_Luna3Session):
    """
    Initializable luna3 sessions.
    """

    def __init__(self, settings: ServiceSettings, lambdaType: str):
        self.settings = settings
        super().__init__(settings=settings, lambdaType=lambdaType)

    def _getLuna3Kwargs(self) -> dict[str, HTTPSettings]:
        kwargs = super()._getLuna3Kwargs()
        if self.settings.additionalServicesUsage.lunaImageStore:
            kwargs["tasksResultStoreSettings"] = HTTPSettings(
                **getServiceClientKwargs(
                    addressSetting=self.settings.taskResultStorage,
                    timeoutSettings=self.settings.taskResultStorageTimeouts,
                )
            )
        if self.settings.additionalServicesUsage.lunaHandlers:
            kwargs["handlersSettings"] = HTTPSettings(
                self.settings.handlersAddress.apiVersion,
                self.settings.handlersAddress.origin,
                **self.settings.handlersTimeouts.asDict(),
                asyncRequest=True,
            )
        return kwargs

    def getRuntimeChecks(self, includeLunaServices: bool = False) -> list[tuple[str, Awaitable]]:
        """
        Returns configured system checks, pairs of (name, coroutine).

        Warnings:
            Check license session in licenseChecker

        Args:
            includeLunaServices: A bool, whether to include checks for luna services.
        """
        checks = super().getRuntimeChecks(includeLunaServices=includeLunaServices)
        if self.tasksResultsStoreSession:
            taskResultClient = self.tasksResultsStoreSession.getClient()
            checks.append(("task", checkService(taskResultClient, includeLunaServices)))
        return checks

    async def probe(self) -> bool:
        """Check that we can connect to the services with current configuration. Can be used without initialization"""

        def generateCheck(client: BaseHTTPAPI, checker, postfix: str = "", **kwargs):
            check = checker(
                client.origin,
                client.api,
                totalTimeout=client.totalTimeout,
                connectTimeout=client.connectTimeout,
                sockReadTimeout=client.sockReadTimeout,
                sockConnectTimeout=client.sockConnectTimeout,
                asyncCheck=True,
                postfix=postfix,
                **kwargs,
            )

            return check

        checks = self.getProbeChecks()
        if self.tasksResultsStoreSession:
            taskResultClient = self.tasksResultsStoreSession.getClient()
            checks.append(
                generateCheck(
                    taskResultClient,
                    checkConnectionToLunaImageStore,
                    postfix="task result",
                    bucket=self.settings.taskResultStorage.bucket,
                )
            )
        return all(await asyncio.gather(*checks))


class AppCtx[TConfig: ServiceSettings](BaseAppCtx):
    """Configurator application context"""

    #: worker redis db context
    workerRedisDBContext: WorkerRedisContext | None
    #: worker tasks
    tasks: dict[BaseSubTask, asyncio.Task]
    #: worker separation tasks
    separationTasks: dict[BaseSubTask, asyncio.Task]
    #: lambda type
    lambdaType: str = "tasks"
    #: asyncio tasks
    tasks: dict
    #: separationTasks: dict
    separationTasks: dict
    #: luna 3 session
    luna3Session: Luna3Session | None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.tasks = {}
        self.separationTasks = {}
        self.luna3session = Luna3Session(settings=self.serviceConfig, lambdaType=self.lambdaType)

    @final
    async def initialize(self):
        """Description see :func:`~_LunaApplication.initialize`"""
        await super().initialize()

        if self.serviceConfig.additionalServicesUsage.lunaImageStore:
            self.initializeSubTaskToImageStore(settings=self.serviceConfig)

        self.initializeBaseWorkerClass(settings=self.serviceConfig, monitoring=self.monitoring)
        self.initializeMonitoringPoints(serviceName=self.lambdaName)
        self.workerRedisDBContext = WorkerRedisContext(self.serviceConfig.redisDB)

        kwargs = {
            "redisContext": self.workerRedisDBContext,
            "luna3Client": self.luna3session.getClient(str(uuid4())),
            "app": self,
        }
        try:
            lambdaTaskClass = importlib.import_module("lambda_main").LambdaTask
            tsk = lambdaTaskClass(**kwargs)
            self.tasks[tsk] = asyncio.create_task(tsk.start(lambdaId=self.lambdaName))
            self.separationTasks[tsk] = asyncio.create_task(tsk.separationStart(lambdaId=self.lambdaName))
        except Exception as exc:
            excArgs = " ".join(getattr(exc, "args", []))
            raise RuntimeError(f"Lambda tasks initialize error: {str(exc)} {excArgs}")

    @final
    async def shutdown(self):
        await super().shutdown()

        await self.workerRedisDBContext.close()

        for task, asyncioTask in self.tasks.items():
            await task.onShutDown()
            asyncioTask.cancel()
        for asyncioTask in self.separationTasks.values():
            asyncioTask.cancel()

    @staticmethod
    def initializeSubTaskToImageStore(settings: ServiceSettings):
        """
        Initialize SubTask to image store class.
        Args:
            settings: task worker settings
        """
        SubTaskToImageStore.initFromConfig(
            resultStoreBucket=settings.taskResultStorage.bucket,
            faceSampleBucket=settings.faceSamplesStorage.bucket,
            bodySampleBucket=settings.bodySamplesStorage.bucket,
            imageOriginBucket=settings.imageOriginStorage.bucket,
            taskResultStorageTimeouts=settings.taskResultStorageTimeouts,
            faceSamplesStorageTimeouts=settings.faceSamplesStorageTimeouts,
            bodySamplesStorageTimeouts=settings.bodySamplesStorageTimeouts,
            imageOriginStorageTimeouts=settings.imageOriginStorageTimeouts,
            imageStoreRequestConcurrency=settings.tasksToImageStoreRequestsConcurrency,
            taskExternalArchiveTimeouts=settings.taskExternalArchiveTimeouts,
        )

    @staticmethod
    def initializeBaseWorkerClass(settings: ServiceSettings, monitoring: LunaMonitoringManager | None) -> None:
        """
        Initialize base SubTask worker class.
        Args:
            settings: task worker settings
            monitoring: luna monitoring
        """
        BaseSubTask.initFromConfig(
            maxErrorCountPerTask=settings.maxErrorCountPerTask,
            monitoring=monitoring,
            limits=settings.platformLimits,
            facesRequestsConcurrency=settings.tasksToFacesRequestsConcurrency,
            handlersRequestConcurrency=settings.tasksToHandlersConcurrency,
            matcherRequestConcurrency=settings.tasksToMatcherConcurrency,
            externalApiAddress=settings.externalApiAddress,
            lisObjectsAddress=settings.objectsOriginStorage,
            lisEnabled=settings.additionalServicesUsage.lunaImageStore,
        )

    @staticmethod
    def initializeMonitoringPoints(serviceName: str):
        """
        Initialize monitoring points classes.
        Args:
            serviceName: custom service name
        """
        WorkerRequestPoint.initialize(serviceName=serviceName)
        SubtaskMonitoringBasePoint.initialize(serviceName=serviceName)


class LambdaTasksApp(BaseLambdaApp):
    """Tasks lambda app"""

    # service context
    ctxCls = AppCtx
    # service config
    configCls = ServiceSettings
